<?php
/*
* Copyright (c) 2018-2021 Atipay, (https://www.atipay.net)

* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
* */

	//load classes init method
	add_action('init', array('PMProGateway_atipay', 'init'));


if(class_exists('PMProGateway')){


	class PMProGateway_atipay extends PMProGateway
    {
        private static $atipay_logo = "https://link.atipay.net/___atipay___/atipay.png";
        private static $callback_params = array();

        function PMProGateway($gateway = NULL)
        {
            $this->gateway = $gateway;
            return $this->gateway;
        }

        /**
         * Run on WP init
         *
         * @since 1.8
         */
        static function init()
        {
            //make sure atipay is a gateway option
            add_filter('pmpro_gateways', array('PMProGateway_atipay', 'pmpro_gateways'));

            //add fields to payment settings
            add_filter('pmpro_payment_options', array('PMProGateway_atipay', 'pmpro_payment_options'));
            add_filter('pmpro_payment_option_fields', array('PMProGateway_atipay', 'pmpro_payment_option_fields'), 10, 2);

            //add some fields to edit user page (Updates)
            add_action('pmpro_after_membership_level_profile_fields', array('PMProGateway_atipay', 'user_profile_fields'));
            add_action('profile_update', array('PMProGateway_atipay', 'user_profile_fields_save'));

            //updates cron
            add_action('pmpro_activation', array('PMProGateway_atipay', 'pmpro_activation'));
            add_action('pmpro_deactivation', array('PMProGateway_atipay', 'pmpro_deactivation'));
            add_action('pmpro_cron_atipay_subscription_updates', array('PMProGateway_atipay', 'pmpro_cron_atipay_subscription_updates'));

            $gateway = pmpro_getOption("gateway");
            if ($gateway == "atipay") {
                add_filter('pmpro_checkout_default_submit_button', array('PMProGateway_atipay', 'pmpro_checkout_default_submit_button'));
                add_filter('pmpro_required_billing_fields', ['PMProGateway_atipay', 'pmpro_required_billing_fields']);
                add_action('pmpro_checkout_preheader', array('PMProGateway_atipay', 'pmpro_checkout_preheader'));
                add_filter('pmpro_include_billing_address_fields', '__return_false');
                add_filter('pmpro_include_payment_information_fields', '__return_false');
                add_filter('pmpro_checkout_before_change_membership_level', array('PMProGateway_atipay', 'pmpro_checkout_before_change_membership_level'), 10, 2);
                add_action('wp_ajax_atipay-rd', ['PMProGateway_atipay', 'pmpro_wp_ajax_atipay_rd']);
            }
        }

        /**
         * Remove required billing fields.
         *
         * @since 1.8
         */
        public static function pmpro_required_billing_fields($fields)
        {
            unset($fields['bfirstname']);
            unset($fields['blastname']);
            unset($fields['baddress1']);
            unset($fields['bcity']);
            unset($fields['bstate']);
            unset($fields['bzipcode']);
            unset($fields['bphone']);
            unset($fields['bemail']);
            unset($fields['bcountry']);
            unset($fields['CardType']);
            unset($fields['AccountNumber']);
            unset($fields['ExpirationMonth']);
            unset($fields['ExpirationYear']);
            unset($fields['CVV']);

            return $fields;
        }

        static function pmpro_checkout_default_submit_button($show)
        {
            global $pmpro_requirebilling;
            ?>

            <img src="<?php echo self::$atipay_logo; ?>" border="0" style="width:45px; height:auto; ">
            <br/>
            <span id="pmpro_submit_span">

            <input type="hidden" name="submit-checkout" value="1"/>
            <input type="submit"
                   class="<?php echo pmpro_get_element_class('pmpro_btn pmpro_btn-submit-checkout', 'pmpro_btn-submit-checkout'); ?>"
                   value="<?php if
                   ($pmpro_requirebilling) {
                       _e('پرداخت با شبکه همراه آتی‌پی', 'paid-memberships-pro');
                   } else {
                       _e('Submit and Confirm', 'paid-memberships-pro');
                   } ?> &raquo;"
            />
            </span>

            <?php
            //don't show the default
            return false;
        }

        /**
         * Instead of change membership levels, get Atipay Token
         *
         * @param int $user_id
         * @param \MemberOrder $morder
         *
         * @since 1.8
         */
        static function pmpro_checkout_before_change_membership_level($user_id, $morder)
        {
            if (empty($morder)) {
                return;
            }

            $morder->user_id = $user_id;
            $morder->status = '';
            $morder->saveOrder();
            do_action("pmpro_before_send_to_atipay", $user_id, $morder);
            $morder->Gateway->getAtipayToken($morder);
        }

        /**
         * Send the data/order to Atipay
         *
         * @param \MemberOrder $order
         */
        function getAtipayToken(& $order)
        {
            if (empty($order)) {
                return;
            }

            $apikey = pmpro_getOption('atipay_apikey');
            $currency = pmpro_getOption('atipay_currency');

            $order_id = $order->code;
            $amount = intval($order->total);
            if ($currency == 'T') {
                $amount = $amount * 10;
            }

            $rd = admin_url('admin-ajax.php') . "?action=atipay-rd&order_id=$order_id";

            $token_params = array(
                'apiKey' => $apikey,
                'redirectUrl' => $rd,
                'invoiceNumber' => $order_id,
                'amount' => $amount,
                'description' => 'order ID #' . $order_id,
            );

            $r = fn_pmpro_atipay_get_token($token_params);
            if ($r['success'] == 1) {
                $order->atipayStatus = 'pending';
                $order->saveOrder();
                $this->sendToAtipay($r['token']);
            } else {
                $error_message = $r['errorMessage'];
                self::payment_failed($order, $error_message);
            }
        }


        static function pmpro_wp_ajax_atipay_rd()
        {
            self::$callback_params = $_POST;
            $params = self::$callback_params;
            $order_id = $_GET['order_id'];
            $apikey = pmpro_getOption('atipay_apikey');
            $currency = pmpro_getOption('atipay_currency');
            $morder = new MemberOrder($order_id);
            $amount = $morder->total;
            if ($currency == 'T') {
                $amount = $amount * 10;
            }

            if (isset($order_id) && !empty($order_id)) {
                if ($order_id == $params['reservationNumber']) {
                    $result = fn_pmpro_check_callback_data($params);
                    if ($result['success']) { //will verify here
                        $verify_params = array('apiKey' => $apikey,
                            'referenceNumber' => $params['referenceNumber']
                        );
                        $r = fn_pmpro_atipay_verify_payment($verify_params, $amount);
                        if ($r['success'] == 0) { //veriy failed
                            $error = $r['errorMessage'];
                            self::payment_failed($morder, $error);
                        } else { //success
                            self::processAtipay($morder);
                        }
                    } else {
                        $error = $result['error'];
                        self::payment_failed($morder, $error);
                    }
                } else {
                    die('Invalid callback parameters');
                }
            } else {
                die('Invalid callback url');
            }
        }


        static function payment_failed(&$morder, $error)
        {
            $order_id = $morder->code;
            $morder->error = $error;
            $morder->notes = $error;
            $morder->status = 'failed';
            $morder->saveOrder();
            $main_url = get_site_url();
            $_url = $main_url . "/membership-account/membership-invoice/?invoice=$order_id";
            header("Location: $_url");
            die();
        }


        /**
         * Send the data/order to Atipay
         *
         * @param $token
         */
        function sendToAtipay($token)
        {
            $form = fn_pmpro_atipay_redirect_to_psp_form($token, 'atipay_membership_payment');
            echo $form;
            die();
        }


        static function processAtipay(&$morder)
        {
            $morder->getMembershipLevel();
            $morder->getUser();
            if (isset($morder->InitialPayment) && floatval($morder->InitialPayment) == 0) {
                if (!pmpro_isLevelTrial($morder->membership_level)) {
                    //subscription will start today with a 1 period trial (initial payment charged separately)
                    $morder->ProfileStartDate = date("Y-m-d") . "T0:0:0";
                    $morder->TrialBillingPeriod = $morder->BillingPeriod;
                    $morder->TrialBillingFrequency = $morder->BillingFrequency;
                    $morder->TrialBillingCycles = 1;
                    $morder->TrialAmount = 0;
                    //add a billing cycle to make up for the trial, if applicable
                    if (!empty($morder->TotalBillingCycles))
                        $morder->TotalBillingCycles++;
                } elseif ($morder->InitialPayment == 0 && $morder->TrialAmount == 0) {
                    //it has a trial, but the amount is the same as the initial payment, so we can squeeze it in there
                    $morder->ProfileStartDate = date("Y-m-d") . "T0:0:0";
                    $morder->TrialBillingCycles++;
                    //add a billing cycle to make up for the trial, if applicable
                    if ($morder->TotalBillingCycles)
                        $morder->TotalBillingCycles++;
                } else {
                    $morder->ProfileStartDate = date("Y-m-d", strtotime("+ " . $morder->BillingFrequency . " " . $morder->BillingPeriod, current_time("timestamp"))) . "T0:0:0";
                }
                self::updateLevelAtipay($morder);
            } else {
                if (pmpro_isLevelRecurring($morder->membership_level)) {
                    if (!pmpro_isLevelTrial($morder->membership_level)) {
                        //subscription will start today with a 1 period trial
                        $morder->ProfileStartDate = date("Y-m-d") . "T0:0:0";
                        $morder->TrialBillingPeriod = $morder->BillingPeriod;
                        $morder->TrialBillingFrequency = $morder->BillingFrequency;
                        $morder->TrialBillingCycles = 1;
                        $morder->TrialAmount = 0;
                        if (!empty($morder->TotalBillingCycles))
                            $morder->TotalBillingCycles++;
                    } elseif ($morder->InitialPayment == 0 && $morder->TrialAmount == 0) {
                        $morder->ProfileStartDate = date("Y-m-d") . "T0:0:0";
                        $morder->TrialBillingCycles++;
                        if (!empty($morder->TotalBillingCycles))
                            $morder->TotalBillingCycles++;
                    } else {
                        $morder->ProfileStartDate = date("Y-m-d", strtotime("+ " . self::BillingFrequency . " " . self::BillingPeriod, current_time("timestamp"))) . "T0:0:0";
                    }
                    $morder->ProfileStartDate = apply_filters("pmpro_profile_start_date", $morder->ProfileStartDate, $morder);
                    self::updateLevelAtipay($morder);
                } else {
                    self::updateLevelAtipay($morder);
                }
            }
        }


        static function updateLevelAtipay(&$morder)
        {
            //
            //the following lines copied from twocheckout gateway.
            //

            $morder->membership_level = apply_filters('pmpro_inshandler_level', $morder->membership_level, $morder->user_id);

            //fix expiration date
            if (!empty($morder->membership_level->expiration_number)) {
                $enddate = "'" . date('Y-m-d', strtotime('+ ' . $morder->membership_level->expiration_number . ' ' . $morder->membership_level->expiration_period, current_time('timestamp'))) . "'";
            } else {
                $enddate = 'NULL';
            }

            $morder->getDiscountCode();
            if (!empty($morder->discount_code)) {
                //update membership level
                $morder->getMembershipLevel(true);
                $discount_code_id = $morder->discount_code->id;
            } else {
                $discount_code_id = '';
            }

            //set the start date to current_time('mysql') but allow filters
            $startdate = apply_filters('pmpro_checkout_start_date', "'" . current_time('mysql') . "'", $morder->user_id, $morder->membership_level);

            $custom_level = [
                'user_id' => $morder->user_id,
                'membership_id' => $morder->membership_level->id,
                'code_id' => $discount_code_id,
                'initial_payment' => $morder->membership_level->initial_payment,
                'billing_amount' => $morder->membership_level->billing_amount,
                'cycle_number' => $morder->membership_level->cycle_number,
                'cycle_period' => $morder->membership_level->cycle_period,
                'billing_limit' => $morder->membership_level->billing_limit,
                'trial_amount' => $morder->membership_level->trial_amount,
                'trial_limit' => $morder->membership_level->trial_limit,
                'startdate' => $startdate,
                'enddate' => $enddate,];

            global $pmpro_error;
            if (!empty($pmpro_error)) {
                echo $pmpro_error;
                inslog($pmpro_error);
            }

            if (pmpro_changeMembershipLevel($custom_level, $morder->user_id) !== false) {
                $morder->status = 'success';
                $morder->payment_transaction_id = self::$callback_params['referenceNumber'];
                $morder->subscription_transaction_id = $morder->code;;
                $morder->saveOrder();

                //add discount code use
                if (!empty($discount_code) && !empty($use_discount_code)) {
                    global $wpdb;
                    $wpdb->query("INSERT INTO $wpdb->pmpro_discount_codes_uses (code_id, user_id, order_id, timestamp) VALUES('" . $discount_code_id . "', '" .
                        $morder->user_id . "', '" . $morder->id . "', '" . current_time('mysql') . "')");
                }

                //save first and last name fields
                if (!empty($_POST['first_name'])) {
                    $old_firstname = get_user_meta($morder->user_id, 'first_name', true);
                    if (!empty($old_firstname)) {
                        update_user_meta($morder->user_id, 'first_name', $_POST['first_name']);
                    }
                }
                if (!empty($_POST['last_name'])) {
                    $old_lastname = get_user_meta($morder->user_id, 'last_name', true);
                    if (!empty($old_lastname)) {
                        update_user_meta($morder->user_id, 'last_name', $_POST['last_name']);
                    }
                }
                do_action("pmpro_after_checkout", $morder->user_id, $morder);
                header('Location:' . pmpro_url('confirmation', '?level=' . $morder->membership_id));
                die('');
            }
        }


        /**
         * Code added to checkout preheader.
         *
         * @since 2.1
         */
        static function pmpro_checkout_preheader()
        {
            //nothing..
        }

        /**
         * Make sure atipay is in the gateways list
         *
         * @since 1.8
         */
        static function pmpro_gateways($gateways)
        {
            if (empty($gateways['atipay']))
                $gateways['atipay'] = 'شبکه همراه آتی‌پی';

            return $gateways;
        }

        /**
         * Get a list of payment options that the atipay gateway needs/supports.
         *
         * @since 1.8
         */
        static function getGatewayOptions()
        {
            $options = array(
                'atipay_currency',
                'atipay_apikey',
                'tax_rate',
            );

            return $options;
        }

        /**
         * Set payment options for payment settings page.
         *
         * @since 1.8
         */
        static function pmpro_payment_options($options)
        {
            //get atipay options
            $atipay_options = PMProGateway_atipay::getGatewayOptions();

            //merge with others.
            $options = array_merge($atipay_options, $options);

            return $options;
        }

        /**
         * Display fields for atipay options.
         *
         * @since 1.8
         */
        static function pmpro_payment_option_fields($values, $gateway)
        {

            $atipay_key = isset($values['atipay_apikey']) && !empty($values['atipay_apikey']) ? esc_attr($values['atipay_apikey']) : "";
            $_currency = isset($values['atipay_currency']) && !empty($values['atipay_currency']) ? esc_attr($values['atipay_currency']) : "";

            ?>

            <tr class="pmpro_settings_divider gateway gateway_atipay pmpro_message" <?php if ($gateway != "atipay") { ?>style="display: none;"<?php } ?>>
                <td colspan="2">
                    <p style="text-align: center;">

                        <a target="_blank" href="https://www.atipay.net"><img src="<?php echo self::$atipay_logo; ?>" border="0" style="width:45px; height:auto; "></a>
                        <br/>

                        تنظیمات درگاه شبکه همراه آتی‌پی
                        <br/>
                        نسخه 1.0.0


                    </p>

                </td>
            </tr>
            <tr class="gateway gateway_atipay pmpro_messagepmpro_message" <?php if ($gateway != "atipay") { ?>style="display: none;"<?php } ?>>
                <th><label for="atipay_apikey"> ApiKey</label></th>
                <td><input type="text" id="atipay_apikey" name="atipay_apikey" size="60" value="<?php echo $atipay_key;
                    ?>"/>
                </td>
            </tr>
            <tr class="gateway gateway_atipay " <?php if ($gateway != "atipay") { ?>style="display: none;"<?php } ?>>
                <th><label for="atipay_currency">Currency</label></th>
                <td>
                    <p class="pmpro_message">
                        در صورتیکه از واحد تومان در سایت استفاده می‌کنید در این قسمت تومان را انتخاب کنید.
                        مبلغ ارسالی به درگاه همواره به صورت ریال در نظر گرفته می‌شود.
                    </p>
                    <?php

                    $_r_select = "";
                    $_t_select = "";
                    if ($_currency == 'R') {
                        $_r_select = "selected";
                    }
                    if ($_currency == 'T') {
                        $_t_select = "selected";
                    }

                    ?>
                    <select name="atipay_currency" id="atipay_currency">
                        <option value="R" <?php echo $_r_select; ?> >Rial</option>
                        <option value="T" <?php echo $_t_select; ?>>Toman</option>
                    </select>
                </td>
            </tr>

            <?php
        }


    }
}